package CPS;


import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.text.DecimalFormat;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Scanner;
import java.util.Set;
import java.util.TreeSet;
import java.util.stream.Collectors;

/*The CPS module (run using this class CPS) takes in the output of the SHOD module*/
public class CPS{ 
	
	private String inputPath1; //purchase input 
	private String inputPath2; //click input
	private String outputPath1; //unique items
	private String outputPath2; //cps results
	
	//default constructor 
	public CPS() {
		inputPath1 = null; 
		inputPath2 = null;
	    outputPath1 = null;
	    outputPath2 = null;
	}
	
	//initializing constructor 
	public CPS(String inputPath1, String inputPath2, String outputPath1, String outputPath2) {
		this.inputPath1 = inputPath1;
		this.inputPath2 = inputPath2;
		this.outputPath1 = outputPath1;
		this.outputPath2 = outputPath2;
	}
	
	
	private List<String> readSequence(Scanner sc) {
		List<String> sequences = new ArrayList<>();
		while(sc.hasNextLine()) {
			sequences.add(sc.nextLine().replaceAll("\"", ""));
		}
		return sequences;
	}
	
	
	private double LCSR(String purchaseSeq, String clickSeq) {
		String[] purchaseItems=purchaseSeq.split(",");
		String[] clickItems=clickSeq.split(",");
		
		return ((double) (Math.min(clickItems.length,purchaseItems.length))/(Math.max(purchaseItems.length,clickItems.length)));
	}
	
	
	private double Cosine(String purchaseSeq, String clickSeq) {
		Map<String, Integer> clickMap=new HashMap<String, Integer>();
		Map<String, Integer> purchaseMap = new HashMap<String, Integer>();
		
		double numerator=0, denumX=0, denumY=0, denominator=0;
		
		String[] purchaseItems=purchaseSeq.split(",");
		String[] clickItems=clickSeq.split(",");
		for(int i=0; i<purchaseItems.length; i++) {
			if(purchaseMap.get(purchaseItems[i]) == null)
				purchaseMap.put(purchaseItems[i], 1);
			else
				purchaseMap.put(purchaseItems[i], purchaseMap.get(purchaseItems[i])+1);
		}
		
		for(int i=0; i<clickItems.length; i++) {
			if(clickMap.get(clickItems[i]) == null)
				clickMap.put(clickItems[i], 1);
			else
				clickMap.put(clickItems[i], clickMap.get(clickItems[i])+1);
		}
		for(String key:clickMap.keySet()) {
			if(clickMap.get(key)==null || purchaseMap.get(key)==null) {
				continue;
			}
			
			numerator+=purchaseMap.get(key)*clickMap.get(key);
			
			denumX += Math.pow(clickMap.get(key), 2);
			denumY += Math.pow(purchaseMap.get(key), 2);
		}
		denumX = Math.sqrt(denumX);
		denumY = Math.sqrt(denumY);
		denominator=denumX * denumY;
		return (numerator/denominator);
	}
	
	
	private List<Double> doCPS(List<String> purchaseSequences, List<String> clickSequences) {
		List<Double> scores = new ArrayList<>();
		Iterator<String> itl1=purchaseSequences.iterator();
		Iterator<String> itl2=clickSequences.iterator();
		
		while(itl1.hasNext()&&itl2.hasNext()) {
			String purchaseSequence=itl1.next();
			String clickSequence=itl2.next();
			scores.add( (0.8 * LCSR(purchaseSequence, clickSequence) + 0.2 * Cosine(purchaseSequence, clickSequence)));
		}

		return scores;
	}
	
	
	private void getUniqueItems(List<String> sequences) {
		Set<Integer> uniqueItems = new TreeSet<Integer>();
		for(String sequence : sequences) {
			uniqueItems.addAll(Arrays.stream(sequence.split(",")).map(Integer::parseInt).collect(Collectors.toSet()));
		}
		try {
			FileWriter writer = new FileWriter(outputPath1);
			BufferedWriter bw = new BufferedWriter(writer);
			
			for(Integer item : uniqueItems) {
				bw.write(item+"\n");
			}
			bw.close();
		} catch (IOException e) {
			e.printStackTrace();
		}
	}
	
	
	private void printToFile(List<Double> purchaseClickSequencesScores) {
		try {
			FileWriter writer = new FileWriter(outputPath2);
			BufferedWriter bw = new BufferedWriter(writer);
			for(Double purchaseClickSequenceScore: purchaseClickSequencesScores) {
				DecimalFormat df = new DecimalFormat();
				df.setMaximumFractionDigits(2);
				bw.write(df.format(purchaseClickSequenceScore).toString()+"\n");
			}
			bw.close();
		} catch (IOException e) {
				System.err.format("IOException: %s%n", e);
			}
	}

	
	public void execute () {
		Scanner sc = null;
		try {
			sc = new Scanner(new BufferedReader(new FileReader(inputPath1)));
		} catch (FileNotFoundException e) {
			e.printStackTrace();
		}
	      List<String> click_sequences = readSequence(sc);
	      getUniqueItems(click_sequences);
	      try {
			sc = new Scanner(new BufferedReader(new FileReader(inputPath2)));
		} catch (FileNotFoundException e) {
			e.printStackTrace();
		}
	      List<String> purchase_sequences =readSequence(sc);
	      sc.close();
	      
	      List<Double> purchaseClickSequencesScores = new ArrayList<>();
	      purchaseClickSequencesScores = doCPS(purchase_sequences, click_sequences);
	      
	      printToFile(purchaseClickSequencesScores);
	}
}