/*****************************************************************************************************************************************************************************
 * 
 * 1. INPUT:
 *      (1) The input required to run CPSRunner.java are the output files of the SHOD module. To be able to run this system you will need to run 
 *          SHOD first.
 * 
        (3) Adjust the filepath at line 56.

            String directoryPath = "/home/woddlab/HSPREC_codes/Datasets/";

    2. HOW TO RUN:
        (1) To run the system you will either have to set this up in an IDE or use the command line, whichever is more suitable for your situation

        IDE SETUP:
            (1) Launch an IDE of your choice
            (2) Click on "file" and then open an existing java project and select the HSPREC folder
            (3) Once the project is loaded in, find the CPSRunner and make the necessary adjustments so that the system works on your machine or directory.
                (Change the directory path)
            (4) Now build and run the program!
        
        COMMAND LINE SETUP:
            (1) Open CPSRunner.java with a text editor and make the necessary adjustments so that the system works on your machine or directory.
                (Change the directory path)
            (2) Open up the command line and make your way to:
                ../../HSPREC_codes/src/main/java
            (3) Once you are at its location compile CPSRunner.java
                javac CPSRunner.java
            (4) Now run CPSRunner
                java CPSRunner

    3. DURING RUNTIME:
        (1) Once the program is running you will prompted to select either (1) paper_dataset or (2) Yoochoose
    
    4. OUTPUT:
        (1) Once the program is finished running it will produce 2 new .txt files in the data folder in results. These are the following files that should have been 
            produced:
                cps_results.txt
                cps_uniqueItems.txt
********************************************************************************************************************************************************************************* */
import CPS.*;

import java.util.Scanner;

/*
 * All code is done by Mustafa Sawalha, Xiaoshuai Geng, and Mustafa Alwindawi
 * CPS, WFPPM, and Normalization modules were solely made by Mustafa Alwindawi
 * Documented by Angel Veliz Castro. All documentation is from Raj's paper
 * */
public class CPSRunner {

    public static void main(String[] args) throws Exception {

		String directoryPath = "/home/woddlab/HSPREC_codes/HSPREC/Datasets/";

		boolean valid = true;
		Scanner sc = new Scanner(System.in);

		/*To make things easier instead of changing the directory path each time I mad a switch case that will add the final part depending on the users input*/
		while(valid) {
			String option;
			System.out.println("Which dataset do you want to run?\n1. paper_dataset\n2. Yoochoose\n");
			option = sc.nextLine();
			switch (option) {
				case "1":
					directoryPath = directoryPath + "paper_dataset/";
					valid = false;
					break;
				case "2":
					directoryPath = directoryPath + "Yoochoose/";
					valid = false;
					break;
				default:
					System.out.println("Invalid Input!");
			}
		}

        /* CPS inputs|outputs */
        String cpsInput1 = directoryPath+"hsp_daily_sequential_db.txt";
        String cpsInput2 = directoryPath+"hsc_daily_sequential_db.txt";
        String cpsOutput1 = directoryPath+"cps_uniqueItems.txt";
        String cpsOutput2 = directoryPath+"cps_results.txt";

        /*********************************************************************************************************************************************************************************************************************************/

        long startTime = System.nanoTime();

        System.out.println("Starting CPS...");

        /*********************************************************************************************************************************************************************************************************************************/

		/*CPS: Click Purchase Similarity
        To compute the CPS between click sequence and purhcase sequence of each user, we have used seqeunce similarity and frequency similarity of the two seqeunces.

         Sequence Similarity: It is based on logest common subsequence rate (LCSR)
         Frequency Similarity: First, form distinct sets of items from both click and
        purchase sequential patterns and count the number of items occurring in each
        sequence to form the vectors specifying the number of times a user clicked or
        purchased a particular item
         */
        CPS cps = new CPS(cpsInput1, cpsInput2, cpsOutput1, cpsOutput2);
        cps.execute();

        long endTime = System.nanoTime();
        long totalDuration = (endTime-startTime)/1000000;

        System.out.println("\n\nCPS has successfully finished in "+totalDuration+" (ms)\n");
    }

}
