/*****************************************************************************************************************************************************************************
 * 
 * 1. INPUT:
 *      (1) To run this program you are going to need the results from the SHOD module.

        (2) Adjust the filepath at lines 68.

            String directoryPath = "/home/woddlab/HSPREC_codes/Datasets/";

    2. HOW TO RUN:
        (1) To run the system you will either have to set this up in an IDE or use the command line, whichever is more suitable for your situation

        IDE SETUP:
            (1) Launch an IDE of your choice
            (2) Click on "file" and then open an existing java project and select the HSPREC folder
            (3) Once the project is loaded in, find the GSPRunner and make the necessary adjustments so that the system works on your machine or directory.
                (Change the directory path)
            (4) Now build and run the program!
        
        COMMAND LINE SETUP:
            (1) Open GSPRunner.java with a text editor and make the necessary adjustments so that the system works on your machine or directory.
                (Change the directory path)
            (2) Open up the command line and make your way to where the GSP Module is located. Similar to this:
                ../../HSPREC_codes/src/main/java
            (3) Once you are at its location compile GSPRunner.java
                javac GSPRunner.java
            (4) Now run GSPRunner
                java GSPRunner

    3. DURING RUNTIME:
        (1) Once the program is running you will prompted to select either (1) paper_dataset or (2) Yoochoose
        (2) You will be prompted to enter a minimum support value which has to be a fraction between 0 and 1 (e.g. 0.5 for 50%), you will also be asked to enter the number of lines(records) the GSP module
 	        should read. This value is the number of lines (records) found in the hsp_daily_sequential_db.txt file produced by the SHOD module.
    
    4. OUTPUT:
        (1) Once the program is finished running it will produce 1 new .txt files in the selected Dataset folder in results. These are the following files that should have been 
            produced:
                frequent_sequence.txt
********************************************************************************************************************************************************************************* */
import GSP.main.*;

import java.util.Scanner;

/*
 * All code is done by Mustafa Sawalha, Xiaoshuai Geng, and Mustafa Alwindawi
 * CPS, WFPPM, and Normalization modules were solely made by Mustafa Alwindawi
 * Documented by Angel Veliz Castro. All documentation is from Raj's paper
 * */
public class GSPRunner {

    public static void main(String[] args) throws Exception {

		String directoryPath = "/home/woddlab/HSPREC_codes/HSPREC/Datasets/";

		float minSupport = 0;
		int numOfLines = 0;
		int minsup_lines = 0;

		boolean valid = true;
		Scanner sc = new Scanner(System.in);

		/*To make things easier instead of changing the directory path each time I mad a switch case that will add the final part depending on the users input*/
		while(valid) {
			String option;
			System.out.println("Which dataset do you want to run?\n1. paper_dataset\n2. Yoochoose\n");
			option = sc.nextLine();
			switch (option) {
				case "1":
					directoryPath = directoryPath + "paper_dataset/";
					System.out.println("Enter the minimum support. Must be a fraction between 0 and 1 (e.g. 0.5 for 50%)");
					minSupport = sc.nextFloat();
					System.out.println("Enter the number of lines (Records) the GSP module should read. (No more than 4)");
					numOfLines = sc.nextInt();

					minsup_lines = (int)(minSupport * numOfLines);
					//System.out.println(minSupport);
					//System.out.println(numOfLines);
					//System.out.println(minsup_lines);
					valid = false;
					break;
				case "2":
					directoryPath = directoryPath + "Yoochoose/";

					System.out.println("Enter the minimum support. Must be a fraction between 0 and 1 (e.g. 0.5 for 50%)");
					minSupport = sc.nextFloat();
					System.out.println("Enter the number of lines (Records) the GSP module should read. (No more than 102079)");
					numOfLines = sc.nextInt();

					minsup_lines = (int)(minSupport * numOfLines);
					//System.out.println(minSupport);
					//System.out.println(numOfLines);
					//System.out.println(minsup_lines);
					valid = false;
					break;
				default:
					System.out.println("Invalid Input!");
			}
		}


        /* GSP inputs|outputs */
        String gspInput = directoryPath +"hsp_daily_sequential_db.txt";
        String gspOutput = directoryPath +"frequent_sequence.txt";

        /*********************************************************************************************************************************************************************************************************************************/

        long startTime = System.nanoTime();

        System.out.println("Starting GSP...");

        /*********************************************************************************************************************************************************************************************************************************/

		/*GSP: Generalized Sequential Pattern Mining*/
		System.out.println("\nNow Executing module 2: GSP...");
		GSPMainRunner gsp = new GSPMainRunner(gspInput, gspOutput, minsup_lines, numOfLines);
		gsp.execute();

        /*********************************************************************************************************************************************************************************************************************************/

        long endTime = System.nanoTime();
        long totalDuration = (endTime-startTime)/1000000;

        System.out.println("\n\nGSP has successfully finished in "+totalDuration+" (ms)\n");
    }

}
