/*****************************************************************************************************************************************************************************
 * This is the Mainrunner for the HSPREC system. The system will run all of the modules (CPS, GSP, SHOD, Normalization, SPR, WFPPM).
 *
 * 1. INPUT:
 *      (1) There are two required files required for this system to run. You will need the ClickData and the PurchaseData, both of these
 *          files are .txt files. Within the HSPREC folder you can find all the modules along with a folder called Datasets. Within Datasets
 *          there will be 3 folders to seperate the sizes of data being used. The small data folder will contain the data exracted from Raj's
 *          paper. The other two folder will be empty and will require you to add your own Click and Purhcase data. Sort your data appropriately
 *          to avoid confusion.
 *      (2) The Click and Purchase file must be formatted like the following, 
 *      Purchase(userid:string, productid:string, rating:real, timestamps:date) otherwise the 
 *      system will crash  while trying to read the data.
 *
 *          #########################################################
 *          Userid	                ProductID	Rating	 Timestamps
 			A036147939NFPC389VLK	B003TSTSOO	5.0	     03 11, 2014
 			A036147939NFPC389VLK	B003TSXHY6	5.0	     03 11, 2014
 			A036147939NFPC389VLK	B0049P1VHS	5.0	     03 12, 2014
 			A036147939NFPC389VLK	B0049P1ZZQ	5.0	     03 12, 2014
 			#########################################################

 Ensure that your files are properly named ClickData.txt and PurchaseData.txt

 (3) Adjust the filepath at line 97. Make sure that it leads to the Dataset folder not a folder within datasets.
 It will look similar to this:
 /home/woddlab/HSPREC_codes/Datasets/

 2. HOW TO RUN:
 (1) To run the system you will either have to set this up in an IDE or use the command line, whichever is more suitable for your situation

 IDE SETUP:
 (1) Launch an IDE of your choice (eg. Eclipse and IntelliJ)
 (2) Click on "file" and then open an existing java project and select the HSPREC file
 (3) Once the project is loaded in, find the Mainrunner and make the necessary adjustments 
 (examples given next) so that the system works on your machine or directory.
 (Change the directory path, make sure that the click and purchase data are properly formated and named correctly)
 (4)Adjust the number of people or who clicked and did not purchase which is at line 166 (this information
 is used in the WFPPM module and a simple program can be used to find this information 
 on the clickdata before running the mainrunner).
 (5) Now build and run the program!

 COMMAND LINE SETUP:
 (1) Open MainRunner.java with a text editor and make the necessary adjustments so that the system works on your machine or directory.
 (Change the directory path, make sure that the click and purchase data are properly formated and named correctly)
 (2) Still while in the editor, adjust the number of users who only clicked and did not purchase to the martix at line 168
 (3) Open up the command line and make your way to where the HSPREC system is located. Similar to this:
 /Users/Angel/IdeaProjects/HSPREC/src/
 (4) Once you are at its location compile MainRunner.java
 javac MainRunner.java
 (5) Now run MainRunner
 java MainRunner

 3. DURING RUNTIME:
 (1) Once the program is running you will prompted to select either (1) paper_dataset or (2) Yoochoose
 (2) You will be prompted to enter a minimum support value which has to be a fraction between 0 and 1 (e.g. 0.5 for 50%), you will also be asked to enter the number of lines(records) the GSP module
 	 should read. This value is the number of lines (records) found in the hsp_daily_sequential_db.txt file produced by the SHOD module.

 4. OUTPUT:
 (1) Once the program is finished running it will produce 14 new .txt files in the selected Dataset folder. These are the following files that should have been
 produced:
 clicks_hsc_intermediate.txt
 cps_results.txt
 cps_uniqueItems.txt
 EditedClickData.txt
 frequent_sequence.txt
 hsc_daily_sequential_db.txt
 hsp_daily_sequential_db.txt
 purchase_hsp_intermediate.txt
 sequenceRules.txt
 user_item_matrix_normalized.txt
 user_item_matrix_updated.txt
 user_item_matrix_WFPPM_results.txt
 user_itemFrequency_matrix.txt
 WFPPM_results.txt
 **********************************************************************************************************************************************************************************/
import SHOD.*;
import GSP.main.*;
import CPS.*;
import SPR.*;
import WFPPM.*;
import Normalization.*;

import java.util.Scanner;

/*
 * All code is done by Mustafa Sawalha, Xiaoshuai Geng, and Mustafa Alwindawi 
 * CPS, WFPPM, and Normalization modules were solely made by Mustafa Alwindawi
 * */
public class MainRunner {

	public static void main(String[] args) throws Exception {

		/*
		 * To run this program, simply change line 24 variable "directoryPath" to where you have your directory that will hold all the files.
		 * The files you need to start this program are your raw and click and purchase data (lines 40 and 41)
		 */

		String directoryPath = "/home/woddlab/HSPREC_codes/HSPREC/Datasets/";

		float minSupport = 0;
        int numOfLines = 0;
		int minsup_lines = 0;
		String option = "";

        boolean valid = true;
        Scanner sc = new Scanner(System.in);

        /*To make things easier instead of changing the directory path each time I mad a switch case that will add the final part depending on the users input*/
        while(valid) {
			System.out.println("Which dataset do you want to run?\n1. paper_dataset\n2. Yoochoose\n");
			option = sc.nextLine();
			switch (option) {
				case "1":
					directoryPath = directoryPath + "paper_dataset/";
					valid = false;
					break;
				case "2":
					directoryPath = directoryPath + "Yoochoose/";
					valid = false;
					break;
				default:
					System.out.println("Invalid Input!");
			}
		}
		
		/* SHOD inputs|outputs */
		String usersThatClickedAndPurchasedInput1 = directoryPath+"ClickData.txt";
		String usersThatClickedAndPurchasedInput2 = directoryPath+"PurchaseData.txt";
		String usersThatClickedAndPurchasedOutput = directoryPath+"EditedClickData.txt";
		
		String shodClicksInput1 = directoryPath+"ClickData.txt";
		String shodClicksOutput1 = directoryPath+"clicks_hsc_intermediate.txt";
		String shodClicksInput2 = directoryPath+"clicks_hsc_intermediate.txt";
		String shodClicksOutput2 = directoryPath+"hsc_daily_sequential_db.txt";
		
		String shodPurchasesInput1 = directoryPath+"PurchaseData.txt";
		String shodPurchasesOutput1 = directoryPath+"purchases_hsp_intermediate.txt";
		String shodPurchasesInput2 = directoryPath+"purchases_hsp_intermediate.txt";
		String shodPurchasesOutput2 = directoryPath+"hsp_daily_sequential_db.txt";
		String userItemFrequencyMatrixOutput = directoryPath+"user_itemFrequency_matrix.txt";
		
		/* GSP inputs|outputs */
		String gspInput = directoryPath+"hsp_daily_sequential_db.txt";
		String gspOutput = directoryPath+"frequent_sequence.txt";

		/* CPS inputs|outputs */
		String cpsInput1 = directoryPath+"hsp_daily_sequential_db.txt";
		String cpsInput2 = directoryPath+"hsc_daily_sequential_db.txt";
		String cpsOutput1 = directoryPath+"cps_uniqueItems.txt";
		String cpsOutput2 = directoryPath+"cps_results.txt";

		/* SPR inputs|outputs */
		String sprInput1 = directoryPath+"frequent_sequence.txt";
		String sprInput2 = directoryPath+"user_itemFrequency_matrix.txt";
		String sprOutput1 = directoryPath+"sequenceRules.txt";
		String sprOutput2 = directoryPath+"user_item_matrix_updated.txt";

		/* WFPPM inputs|outputs */
		String wfppmInput1 = directoryPath+"hsp_daily_sequential_db.txt";
		String wfppmInput2 = directoryPath+"cps_results.txt";
		String wfppmInput3 = directoryPath+"cps_uniqueItems.txt";
		String wfppmInput4 = directoryPath+"PurchaseData.txt";
		String wfppmInput5 = directoryPath+"ClickData.txt";
		String wfppmInput6 = directoryPath+"user_item_matrix_updated.txt";
		int wfppmInput7 = 1; //num of people that only clicked and did not purchase, added to the matrix. Change depending on your need
		String wfppmOutput1 = directoryPath+"WFPPM_results.txt";
		String wfppmOutput2 = directoryPath+"user_item_matrix_WFPPM_result.txt";

		/* Normalization inputs|outputs */
		String normalizationInput = directoryPath+"user_item_matrix_WFPPM_result.txt";
		String normalizationOutput = directoryPath+"user_item_matrix_normalized.txt";
		
		/*********************************************************************************************************************************************************************************************************************************/
		
		long startTime = System.nanoTime();
		
		System.out.println("Starting HSPREC...");
		
		/*********************************************************************************************************************************************************************************************************************************/

		/*

		SHOD:

		The proposed sequential historical pattern generation (SHOD) module takes
		  historical (click or purchase database) data as input and produces periodic (daily,
		  weekly, monthly) sequential (click or purchase) database as output. This algorithm is based on checking timestamp between items
		  to form periodic sequences if the difference between purchase times by the same
		  user are within 24 h, 24 * 7 h respectively */

		System.out.println("\nExecuting module 1: SHOD...");
		UsersThatClickedAndPurchasedGenerator usersThatClickedAndPurchasedGenerator = new UsersThatClickedAndPurchasedGenerator(usersThatClickedAndPurchasedInput1, usersThatClickedAndPurchasedInput2, usersThatClickedAndPurchasedOutput);
		usersThatClickedAndPurchasedGenerator.execute();
		UserItemFrequencyMatrixGenerator userItemFrequencyMatrixGenerator = new UserItemFrequencyMatrixGenerator(shodPurchasesInput1, userItemFrequencyMatrixOutput);
		userItemFrequencyMatrixGenerator.execute();
		
     	//Clicks
		SequenceDBMain clicksIntermediateDB = new SequenceDBMain(shodClicksInput1, shodClicksOutput1);
		clicksIntermediateDB.generateIntermediateDB();
		StrictMultisequenceHSP clicksDailySequentialDB = new StrictMultisequenceHSP(shodClicksInput2, shodClicksOutput2);
		clicksDailySequentialDB.generateDailySequentialDB();

		//Purchases
		SequenceDBMain purhcasesIntermediateDB = new SequenceDBMain(shodPurchasesInput1, shodPurchasesOutput1);
		purhcasesIntermediateDB.generateIntermediateDB();
		StrictMultisequenceHSP purchasesDailySequentialDB = new StrictMultisequenceHSP(shodPurchasesInput2, shodPurchasesOutput2);
		purchasesDailySequentialDB.generateDailySequentialDB();
	

		/*********************************************************************************************************************************************************************************************************************************/

		/*GSP: Generalized Sequential Pattern Mining*/

		System.out.println("Enter the minimum support. Must be a fraction between 0 and 1 (e.g. 0.5 for 50%)");
		minSupport = sc.nextFloat();
		if(option.equals("1")) {
			System.out.println("Enter the number of lines (Records) the GSP module should read. (No more than 4)");
		}
		else {
			System.out.println("Enter the number of lines (Records) the GSP module should read. (No more than 102079)");
		}

		numOfLines = sc.nextInt();

		minsup_lines = (int)(minSupport * numOfLines);

		//System.out.println(minSupport);
		//System.out.println(numOfLines);
		//System.out.println(minsup_lines);
		System.out.println("\nNow Executing module 2: GSP...");
		GSPMainRunner gsp = new GSPMainRunner(gspInput, gspOutput, minsup_lines, numOfLines);
		gsp.execute();

		/*********************************************************************************************************************************************************************************************************************************/


		/*CPS: Click Purchase Similarity
        To compute the CPS between click sequence and purhcase sequence of each user, we have used seqeunce similarity and frequency similarity of the two seqeunces.

         Sequence Similarity: It is based on logest common subsequence rate (LCSR)
         Frequency Similarity: First, form distinct sets of items from both click and
        purchase sequential patterns and count the number of items occurring in each
        sequence to form the vectors specifying the number of times a user clicked or
        purchased a particular item
         */
		System.out.println("\nNow Executing module 3: CPS...");
		CPS cps = new CPS(cpsInput1, cpsInput2, cpsOutput1, cpsOutput2);
		cps.execute();

		/*********************************************************************************************************************************************************************************************************************************/


		/*SPR: Sequential Pattern Rule
		  Sequential Pattern Rule (SPR) is based on the use of frequent sequential patterns
		  created from periodic sequential database. Thus, input of SPR is periodic historical sequential database and output is recommended possible purchase items
		  derived from generated sequential patterns rules.
		  The major steps in SPR are:

		  Frequent sequences generation: Generates frequent sequences using Generalized Sequential Pattern Mining (GSP) algorithm

		  Rule generation: Represent frequent sequences in the form of U-click →
		  U-purchase, where the left-hand side of the rule refers to clicked item set while
		  the right-hand side is the recommended item to be purchased.


		  Rule section: Assume we are ony interested in rules that satisfy the following
		  criteria: (1) At least two antecedents. (2) Confidence > 50%. (3) Select one
		  rule with highest confidence
		   **/
		System.out.println("\nNow Executing module 4: SPR...");
		SPR spr = new SPR(sprInput1, sprInput2, sprOutput1, sprOutput2);
		spr.execute();

		/*********************************************************************************************************************************************************************************************************************************/

		/*WFPPM: Weighted Frequent Purchase Pattern Miner
		  Weighted Frequent Purchase Pattern Miner(WFPPM) takes weighted purchase
		  pattern as input  and weighted purchase patterns is created
		  by assigning CPS (click sequence, purchase sequence) value to purchase patterns
		  in consequential bond to generate frequent purchase patterns with weight
		  under user specified minimum threshold as output.

		  Major steps of WFPPM are:

		  Count support of items: Count occurrence of items presented in weighted
		  purchase pattern

		  Calculate weight for individual item: Compute weight of individual item
		  from weighted purchase pattern using CPS module (Eq. (4)) (Can be found in the paper)

		  Test item weight with minimum threshold: Define minimum threshold
		  rating for user, here in our case, minimum threshold = 0.2. So, all rating of
		  item are frequent.
		*/
		System.out.println("\nNow Executing module 5: WFFPM...");
		WFPPM wfppm = new WFPPM(wfppmInput1, wfppmInput2, wfppmInput3, wfppmInput4, wfppmInput5, wfppmInput6, wfppmInput7, wfppmOutput1, wfppmOutput2);
		wfppm.execute();

		/*********************************************************************************************************************************************************************************************************************************/


		/*Normalization in recommendation system helps to predict the level of interest
		  of user on a particular purchased item. Thus, normalization function for a user
		  u’s rating of an item i (r_ui) takes user-item frequency matrix as input
		  and provides the level of user interest between 0 and 1 using unit vector formula
		*/
		System.out.println("\nNow Executing module 6: Normalization...");
		Normalization normalization = new Normalization(normalizationInput, normalizationOutput);
		normalization.execute();

		/*********************************************************************************************************************************************************************************************************************************/

		long endTime = System.nanoTime();
		long totalDuration = (endTime-startTime)/1000000;
		
		System.out.println("\n\nHSPREC has successfully finished in "+totalDuration+" (ms) and is now ready for Collaborative Filtering...\n");
		System.out.println("The results can found in " + directoryPath);
	}

}
