/*****************************************************************************************************************************************************************************
 * This is the NormRunner for the Normalization Module.
 * 
 * 1. INPUT:
 *      (1) To run this program you are going to need the results from the WFPPM as input.

        (2) Adjust the filepath at lines 86-87. Make sure that it leads to the results of WFPPM and Normalization in the data folder. 
            It will look similar to this:

            String directoryPath = "/home/woddlab/HSPREC_codes/Datasets/";

    2. HOW TO RUN:
        (1) To run the system you will either have to set this up in an IDE or use the command line, whichever is more suitable for your situation

        IDE SETUP:
            (1) Launch an IDE of your choice
            (2) Click on "file" and then open an existing java project and select the HSPREC folder.
            (3) Once the project is loaded in, find the NormRunner and make the necessary adjustments so that the system works on your machine or directory.
                (Change the directory path)
            (4) Now build and run the program!
        
        COMMAND LINE SETUP:
            (1) Open NormRunner.java with a text editor and make the necessary adjustments so that the system works on your machine or directory.
                (Change the directory path)
            (2) Open up the command line and make your way to where the Normalization Module is located.
            (3) Once you are at its location compile NormRunner.java
                javac NormRunner.java
            (4) Now run NormRunner
                java NormRunner

     3. DURING RUNTIME:
        (1) Once the program is running you will prompted to select either (1) paper_dataset or (2) Yoochoose
    
    4. OUTPUT:
        (1) Once the program is finished running it will produce 14 new .txt files in the selected Dataset folder. These are the following files that should have been 
            produced:
                user_item_matrix_normalized.txt
********************************************************************************************************************************************************************************* */
import Normalization.*;

import java.util.Scanner;

/*
 * All code is done by Mustafa Sawalha, Xiaoshuai Geng, and Mustafa Alwindawi
 * CPS, WFPPM, and Normalization modules were solely made by Mustafa Alwindawi
 * Documented by Angel Veliz Castro. All documentation is from Raj's paper
 * */
public class NormRunner {

    public static void main(String[] args) throws Exception {

		String directoryPath = "/home/woddlab/HSPREC_codes/HSPREC/Datasets/";

		boolean valid = true;
		Scanner sc = new Scanner(System.in);

		/*To make things easier instead of changing the directory path each time I mad a switch case that will add the final part depending on the users input*/
		while(valid) {
			String option;
			System.out.println("Which dataset do you want to run?\n1. paper_dataset\n2. Yoochoose\n");
			option = sc.nextLine();
			switch (option) {
				case "1":
					directoryPath = directoryPath + "paper_dataset/";
					valid = false;
					break;
				case "2":
					directoryPath = directoryPath + "Yoochoose/";
					valid = false;
					break;
				default:
					System.out.println("Invalid Input!");
			}
		}

        /* Normalization inputs|outputs */
        String normalizationInput = directoryPath+"user_item_matrix_WFPPM_result.txt";
        String normalizationOutput = directoryPath+"user_item_matrix_normalized.txt";

        /*********************************************************************************************************************************************************************************************************************************/

        long startTime = System.nanoTime();

        System.out.println("Starting Normalization...");

        /*********************************************************************************************************************************************************************************************************************************/

		/*Normalization in recommendation system helps to predict the level of interest
		  of user on a particular purchased item. Thus, normalization function for a user
		  u’s rating of an item i (r_ui) takes user-item frequency matrix as input
		  and provides the level of user interest between 0 and 1 using unit vector formula
		*/
        Normalization normalization = new Normalization(normalizationInput, normalizationOutput);
        normalization.execute();

        /*********************************************************************************************************************************************************************************************************************************/

        long endTime = System.nanoTime();
        long totalDuration = (endTime-startTime)/1000000;

        System.out.println("\n\nNormalization has successfully finished in "+totalDuration+" (ms) and is now ready for Collaborative Filtering...\n");
    }

}
