package Normalization;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.util.Map;
import java.util.Scanner;
import java.util.TreeMap;

/*The Normalization module (run using this class Normalization) takes in the output of the WFFPM module*/
public class Normalization {

	private String inputPath;
	private String outputPath;
		
	//default constructor
	public Normalization() {
		inputPath = null;
		outputPath = null;
	}
	
	//initializing constructor
	public Normalization(String inputPath, String outputPath) {
		this.inputPath = inputPath;
		this.outputPath = outputPath;
	}
	
	
	private void printToFile(Map<Integer, Map<String, Double> > userItemFrequencyMatrix) {
		try {
			BufferedWriter bw = new BufferedWriter(new FileWriter(outputPath));
			userItemFrequencyMatrix.forEach((k, v) -> {v.forEach((X,Y) -> {try {
				bw.write(k + " " + X + " " + String.format("%.2f", Y) + "\n");
			} catch (IOException e) {
				e.printStackTrace();
			}
			});});
			bw.close();
		} catch (IOException e1) {
			e1.printStackTrace();
		}
	}
	
	
	private Map<Integer, Map<String, Double>> calculateNormalization(Map<Integer, Map<String, Double>> userItemFrequencyMatrix) {
		Map<Integer, Map<String, Double>> updatedUserItemFrequencyMatrix = new TreeMap<Integer, Map<String, Double>>();
		for(Integer user:userItemFrequencyMatrix.keySet()) {
			Map<String, Double> innerMap = userItemFrequencyMatrix.get(user);
			double denominator = 0;
			double frequency = 0;
			double normalizedFrequency = 0;
			boolean skip = false;
			for(String item: innerMap.keySet()) {
				frequency = innerMap.get(item);
				if(frequency<1)
					skip = true;
				denominator+=Math.pow(frequency, 2);
			}
			if(skip) {
				for(String item: innerMap.keySet()) {
					frequency = innerMap.get(item);
					updatedUserItemFrequencyMatrix.computeIfAbsent(user, k -> new TreeMap<>()).put(item, frequency);
				}
				continue;
			}
			for(String item: innerMap.keySet()) {
				frequency = innerMap.get(item);
				normalizedFrequency = (frequency/Math.sqrt(denominator));
				updatedUserItemFrequencyMatrix.computeIfAbsent(user, k -> new TreeMap<>()).put(item, normalizedFrequency);
			}
		}
		return updatedUserItemFrequencyMatrix;
	}
	
	
	private Map<Integer, Map<String, Double>> readMatrix(Scanner sc){
		Map<Integer, Map<String, Double>> userItemFrequencyMatrix = new TreeMap<Integer, Map<String, Double>>();
		while(sc.hasNextLine()) {
			String[] purchase = sc.nextLine().split(" ");
			userItemFrequencyMatrix.computeIfAbsent(Integer.parseInt(purchase[0]), k -> new TreeMap<>()).put(purchase[1], Double.parseDouble(purchase[2]));
		}
		return userItemFrequencyMatrix;
	}
	

	public void execute() {
		try {
			Scanner sc = new Scanner(new BufferedReader(new FileReader(inputPath)));
			Map<Integer, Map<String, Double> > userItemFrequencyMatrix = readMatrix(sc);
			printToFile(calculateNormalization(userItemFrequencyMatrix));
		} catch (IOException e) {
			e.printStackTrace();
		}
		
		
	}
}