package SHOD;
	
import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.util.Map;
import java.util.Scanner;
import java.util.TreeMap;


public class UserItemFrequencyMatrixGenerator {
	
	private Map<Integer, Map<String, Integer> > userItemFrequencyMatrix;
	private String inputPath;
	private String outputPath;
	
	public UserItemFrequencyMatrixGenerator(){
		inputPath = null;
		outputPath = null;
		userItemFrequencyMatrix = new TreeMap<Integer, Map<String, Integer>>();
	}
	
	public UserItemFrequencyMatrixGenerator(String inputPath, String outputPath){
		this.inputPath = inputPath;
		this.outputPath = outputPath;
		userItemFrequencyMatrix = new TreeMap<Integer, Map<String, Integer>>();
	}
		
	private void add(Integer user, String item) {
			userItemFrequencyMatrix.computeIfAbsent(user, k -> new TreeMap<>()).merge(item, 1, Integer::sum);
	}
		
	private void printToFile(BufferedWriter bw) {
		userItemFrequencyMatrix.forEach((k, v) -> {v.forEach((X,Y) -> {try {
				bw.write(k + " " + X + " " + Y + "\n");
			} catch (IOException e) {
				e.printStackTrace();
			}
		});});
			
		try {
			bw.close();
		} catch (IOException e) {
			e.printStackTrace();
		}
	}
	
	private void readAndAddToMatrix(Scanner sc) {
		while(sc.hasNextLine()) {
			String[] line = sc.nextLine().split("\t");
			add(Integer.parseInt(line[0]), line[1]);
		}
	}
	
	public void execute() {
		try {
			Scanner sc = new Scanner(new BufferedReader(new FileReader(inputPath)));
			BufferedWriter bw = new BufferedWriter(new FileWriter(outputPath));
			readAndAddToMatrix(sc);
			printToFile(bw);
		} catch (IOException e) {
			e.printStackTrace();
		}
	}
}

