/****************************************************************************************************************************************************************************
 *
 * 1. INPUT:
 *      (1) There is two required files required for this system to run. You will need the ClickData and the PurchaseData, both of these
 *          files are .txt files.
 *      (2) The Click and Purchase file must be formatted like the following, otherwise the system will crash while trying to read the data.
 *          The schema of both the click and purchase data is:
 *          {Userid(PK), ProductID, Rating, Timestamp}
 *          #########################################################
 *          Userid	                ProductID	Rating	 Timestamps
            A036147939NFPC389VLK	B003TSTSOO	5.0	     03 11, 2014
            A036147939NFPC389VLK	B003TSXHY6	5.0	     03 11, 2014
            A036147939NFPC389VLK	B0049P1VHS	5.0	     03 12, 2014
            A036147939NFPC389VLK	B0049P1ZZQ	5.0	     03 12, 2014
            #########################################################

 Ensure that your files are properly named ClickData.txt and PurchaseData.txt

 (3) Adjust the filepath at line 80. Make sure that it leads to the Dataset folder not a folder within datasets.
 It will look similar to this:
 String directoryPath = "/home/woddlab/HSPREC_codes/Datasets/";

 2. HOW TO RUN:
 (1) To run the system you will either have to set this up in an IDE or use the command line, whichever is more suitable for your situation

 IDE SETUP:
 (1) Launch an IDE of your choice
 (2) Click on "file" and then open an existing java project and select the HSPREC folder
 (3) Once the project is loaded in, find the SHODRunner and make the necessary adjustments so that the system works on your machine or directory.
 (Change the directory path, make sure that the click and purchase data are properly formated and named correctly)
 (4) Now build and run the program!

 COMMAND LINE SETUP:
 (1) Open SHODRunner.java with a text editor and make the necessary adjustments so that the system works on your machine or directory.
 (Change the directory path, make sure that the click and purchase data are properly formated and named correctly)
 (2) Open up the command line and make your way to:
    ../../HSPREC_codes/src/main/java/
 (4) Once you are at its location compile SHODRunner.java
 javac SHODRunner.java
 (5) Now run SHODRunner
 java SHODRunner

 3. DURING RUNTIME:
 (1) Once the program is running you will prompted to select either (1) paper_dataset or (2) Yoochoose

 4. OUTPUT:
 (1) Once the program is finished running it will produce 6 new .txt files in the selected Dataset folder. These are the following files that should have been
 produced:
 clicks_hsc_intermediate.txt
 EditedClickData.txt
 hsc_daily_sequential_db.txt
 hsp_daily_sequential_db.txt
 purchase_hsp_intermediate.txt
 user_itemFrequency_matrix.txt
 ********************************************************************************************************************************************************************************* */
import SHOD.*;

import java.util.Scanner;

/*
 * All code is done by Mustafa Sawalha, Xiaoshuai Geng, and Mustafa Alwindawi
 * CPS, WFPPM, and Normalization modules were solely made by Mustafa Alwindawi
 * Documented by Angel Veliz Castro. All documentation is from Raj's paper
 * */
public class SHODRunner {

    public static void main(String[] args) throws Exception {

        String directoryPath = "/home/woddlab/HSPREC_codes/HSPREC/Datasets/";

		boolean valid = true;
		Scanner sc = new Scanner(System.in);

		/*To make things easier instead of changing the directory path each time I mad a switch case that will add the final part depending on the users input*/
		while(valid) {
			String option;
			System.out.println("Which dataset do you want to run?\n1. paper_dataset\n2. Yoochoose\n");
			option = sc.nextLine();
			switch (option) {
				case "1":
					directoryPath = directoryPath + "paper_dataset/";
					valid = false;
					break;
				case "2":
					directoryPath = directoryPath + "Yoochoose/";
					valid = false;
					break;
				default:
					System.out.println("Invalid Input!");
			}
		}

        /* SHOD inputs|outputs */
        String usersThatClickedAndPurchasedInput1 = directoryPath+"ClickData.txt";
        String usersThatClickedAndPurchasedInput2 = directoryPath+"PurchaseData.txt";
        String usersThatClickedAndPurchasedOutput = directoryPath+"EditedClickData.txt";

        String shodClicksInput1 = directoryPath+"ClickData.txt";
        String shodClicksOutput1 = directoryPath+"clicks_hsc_intermediate.txt";
        String shodClicksInput2 = directoryPath+"clicks_hsc_intermediate.txt";
        String shodClicksOutput2 = directoryPath+"hsc_daily_sequential_db.txt";

        String shodPurchasesInput1 = directoryPath+"PurchaseData.txt";
        String shodPurchasesOutput1 = directoryPath+"purchases_hsp_intermediate.txt";
        String shodPurchasesInput2 = directoryPath+"purchases_hsp_intermediate.txt";
        String shodPurchasesOutput2 = directoryPath+"hsp_daily_sequential_db.txt";
        String userItemFrequencyMatrixOutput = directoryPath+"user_itemFrequency_matrix.txt";

        long startTime = System.nanoTime();

        System.out.println("Starting SHOD...");

        /*********************************************************************************************************************************************************************************************************************************/

		/*

		SHOD:

		The proposed sequential historical pattern generation (SHOD) module takes
		  historical (click or purchase database) data as input and produces periodic (daily,
		  weekly, monthly) sequential (click or purchase) database as output. This algorithm is based on checking timestamp between items
		  to form periodic sequences if the difference between purchase times by the same
		  user are within 24 h, 24 * 7 h respectively */

        UsersThatClickedAndPurchasedGenerator usersThatClickedAndPurchasedGenerator = new UsersThatClickedAndPurchasedGenerator(usersThatClickedAndPurchasedInput1, usersThatClickedAndPurchasedInput2, usersThatClickedAndPurchasedOutput);
        usersThatClickedAndPurchasedGenerator.execute();
        UserItemFrequencyMatrixGenerator userItemFrequencyMatrixGenerator = new UserItemFrequencyMatrixGenerator(shodPurchasesInput1, userItemFrequencyMatrixOutput);
        userItemFrequencyMatrixGenerator.execute();

        //Clicks
        SequenceDBMain clicksIntermediateDB = new SequenceDBMain(shodClicksInput1, shodClicksOutput1);
        clicksIntermediateDB.generateIntermediateDB();
        StrictMultisequenceHSP clicksDailySequentialDB = new StrictMultisequenceHSP(shodClicksInput2, shodClicksOutput2);
        clicksDailySequentialDB.generateDailySequentialDB();

        //Purchases
        SequenceDBMain purhcasesIntermediateDB = new SequenceDBMain(shodPurchasesInput1, shodPurchasesOutput1);
        purhcasesIntermediateDB.generateIntermediateDB();
        StrictMultisequenceHSP purchasesDailySequentialDB = new StrictMultisequenceHSP(shodPurchasesInput2, shodPurchasesOutput2);
        purchasesDailySequentialDB.generateDailySequentialDB();

        long endTime = System.nanoTime();
        long totalDuration = (endTime-startTime)/1000000;

        System.out.println("\n\nSHOD has successfully finished in "+totalDuration+" (ms)\n");
    }

}
