package SPR;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileWriter;
import java.io.IOException;
import java.io.InputStreamReader;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Scanner;
import java.util.TreeMap;

/*The SPR module (run using this class SPR) takes in the output of the GSP and SHOD modules*/
public class SPR {
	
	private String inputPath1;
	private String inputPath2;
	private String outputPath1;
	private String outputPath2;
	
	//default constructor
	public SPR() {
		inputPath1 = null;
		inputPath2 = null;
		outputPath1 = null;
		outputPath2 = null;
	}
	
	//initializing constructor
	public SPR(String inputPath1, String inputPath2, String outputPath1, String outputPath2) {
		this.inputPath1 = inputPath1;
		this.inputPath2 = inputPath2;
		this.outputPath1 = outputPath1;
		this.outputPath2 = outputPath2;
	}
	
	
	private void printToFile(List<String> results) {
		try {
			BufferedWriter bw = new BufferedWriter(new FileWriter(outputPath1));
			for(String result: results) {
				String[] items = result.split(" ");
				bw.write("(" + items[0] + ") -> (" + items[1] + ")\n");
			}
			bw.close();
		} catch (IOException e) {
			e.printStackTrace();
		}
	}
	

	public static void printMatrixtToFile(Map<Integer, Map<String, Integer>> userItemFrequencyMatrix, BufferedWriter bw) {
		userItemFrequencyMatrix.forEach((k, v) -> {
			v.forEach((X,Y) -> {
				try {
					bw.write(k + " " + X + " " + Y + "\n");
				} catch (IOException e) {
					e.printStackTrace();
				}
			});
		});
	}
	
	
	private List<String> generateRules(Map<String, Integer> map) {
		List<String> results = new ArrayList<>();
		map.forEach((K,V)->{
			map.forEach((X,Y)->{
				if(map.get(K + "," + X) != null && ( (double) map.get(K + "," + X) / V ) >= 0.5) {
					results.add(K +" "+ X );
				}
			});
		});
		return results;
	}
	
	
	private Map<String, Integer> readSequences(Scanner sc){
		Map<String, Integer> map=new HashMap<String, Integer>();  
		while (sc.hasNextLine()) {
			String line = sc.nextLine();
			String sequence = line.substring(line.indexOf('(')+1, line.lastIndexOf(')')).replaceAll(" ","");
			String frequency = line.substring(line.indexOf("frequency: ") + 11, line.indexOf(']'));
			map.put(sequence, Integer.parseInt(frequency));
		}
		return map;
	}
	
	
	private Map<Integer, Map<String, Integer>> applyResultsToMatrix(Map<Integer, Map<String, Integer>> userItemFrequencyMatrix, List<String> results) {
		Map<Integer, Map<String, Integer>> userItemFrequencyMatrixNew = new TreeMap<Integer, Map<String, Integer>>();
		for(String result: results) {
			String[] items = result.split(" ");
			String[] antesedent = items[0].split(",");
			int index=0;
			userItemFrequencyMatrix.forEach((user, innerMap) -> {
				boolean skip=false;
				for(int i=0;i<antesedent.length;i++) {
					if(innerMap.get(antesedent[index]) == null)
						skip=true;
				}
				if(!skip) {
					String[] desendent = items[1].split(",");
					for(int i=0;i<desendent.length;i++)
						userItemFrequencyMatrixNew.computeIfAbsent(user, key -> new TreeMap<>()).put(desendent[i], 1);
				}
			});
		}
		
		userItemFrequencyMatrixNew.forEach((X, Y) -> {
			Y.forEach((K,V) -> {
				userItemFrequencyMatrix.computeIfAbsent(X, k -> new TreeMap<>()).put(K, V);
			});
		});
		
		return userItemFrequencyMatrix;
	}
	
	
	private Map<Integer, Map<String, Integer>> readMatrix(Scanner sc){
		Map<Integer, Map<String, Integer>> userItemFrequencyMatrix = new TreeMap<Integer, Map<String, Integer>>();
		while(sc.hasNextLine()) {
			String[] purchase = sc.nextLine().split(" ");
			userItemFrequencyMatrix.computeIfAbsent(Integer.parseInt(purchase[0]), k -> new TreeMap<>()).put(purchase[1], Integer.parseInt(purchase[2]));
		}
		
		
		return userItemFrequencyMatrix;
	}
	
	
	public void execute() {
		
		try {
			Scanner sc = new Scanner (new File(inputPath1));
			List<String> results = generateRules(readSequences(sc)); 
			printToFile(results);
			sc.close();
			sc = new Scanner(new File(inputPath2));
			Map<Integer, Map<String, Integer>> userItemFrequencyMatrix = applyResultsToMatrix(readMatrix(sc), results);
			BufferedWriter bw =  new BufferedWriter(new FileWriter(outputPath2));
			printMatrixtToFile(userItemFrequencyMatrix, bw);
			bw.close();
		} catch (IOException e) {
			e.printStackTrace();
		}
	}
}