/*****************************************************************************************************************************************************************************
 * 
 * 1. INPUT:
 *      (1) There is two required files required for this system to run.You will need frequent_sequence.txt and user_item_matrix_updated.txt. These files are the outputs of SHOD and GSP.

        (2) Adjust the filepath at line 97.
            It will look similar to this:
            String directoryPath = "/home/woddlab/HSPREC_codes/Datasets/";

    2. HOW TO RUN:
        (1) To run the system you will either have to set this up in an IDE or use the command line, whichever is more suitable for your situation

        IDE SETUP:
            (1) Launch an IDE of your choice
            (2) Click on "file" and then open an existing java project and select the HSPREC folder.
            (3) Once the project is loaded in, find the SPRunner and make the necessary adjustments so that the system works on your machine or directory.
                (Change the directory path)
            (4) Now build and run the program!
        
        COMMAND LINE SETUP:
            (1) Open SPRunner.java with a text editor and make the necessary adjustments so that the system works on your machine or directory.
                (Change the directory path)
            (2) Open up the command line and make your way to where the HSPREC_codes system is located. Similar to this:
                /home/woddlab/Angel_Veliz_Castro/HSPREC_codes/src/
            (3) Once you are at its location compile SPRunner.java
                javac SPRunner.java
            (4) Now run SPRunner
                java SPRunner

     3. DURING RUNTIME:
        (1) Once the program is running you will prompted to select either (1) paper_dataset or (2) Yoochoose
    
    4. OUTPUT:
        (1) Once the program is finished running it will produce 2 new .txt files in the data folder in results. These are the following files that should have been 
            produced:
                sequenceRules.txt
                user_item_matrix_updated.txt
********************************************************************************************************************************************************************************* */
import SPR.*;

import java.util.Scanner;

/*
 * All code is done by Mustafa Sawalha, Xiaoshuai Geng, and Mustafa Alwindawi
 * CPS, WFPPM, and Normalization modules were solely made by Mustafa Alwindawi
 * Documented by Angel Veliz Castro. All documentation is from Raj's paper
 * */
public class SPRunner {

    public static void main(String[] args) throws Exception {

		String directoryPath = "/home/woddlab/HSPREC_codes/HSPREC/Datasets/";

		boolean valid = true;
		Scanner sc = new Scanner(System.in);

		/*To make things easier instead of changing the directory path each time I mad a switch case that will add the final part depending on the users input*/
		while(valid) {
			String option;
			System.out.println("Which dataset do you want to run?\n1. paper_dataset\n2. Yoochoose\n");
			option = sc.nextLine();
			switch (option) {
				case "1":
					directoryPath = directoryPath + "paper_dataset/";
					valid = false;
					break;
				case "2":
					directoryPath = directoryPath + "Yoochoose/";
					valid = false;
					break;
				default:
					System.out.println("Invalid Input!");
			}
		}

       
        /* SPR inputs|outputs */
        String sprInput1 = directoryPath+"frequent_sequence.txt";
        String sprInput2 = directoryPath+"user_itemFrequency_matrix.txt";
        String sprOutput1 = directoryPath+"sequenceRules.txt";
        String sprOutput2 = directoryPath+"user_item_matrix_updated.txt";

        /*********************************************************************************************************************************************************************************************************************************/

        long startTime = System.nanoTime();

        System.out.println("Starting SPR...");

        /*********************************************************************************************************************************************************************************************************************************/


		/*SPR: Sequential Pattern Rule
		  Sequential Pattern Rule (SPR) is based on the use of frequent sequential patterns
		  created from periodic sequential database. Thus, input of SPR is periodic historical sequential database and output is recommended possible purchase items
		  derived from generated sequential patterns rules.
		  The major steps in SPR are:

		  Frequent sequences generation: Generates frequent sequences using Generalized Sequential Pattern Mining (GSP) algorithm

		  Rule generation: Represent frequent sequences in the form of U-click →
		  U-purchase, where the left-hand side of the rule refers to clicked item set while
		  the right-hand side is the recommended item to be purchased.


		  Rule section: Assume we are ony interested in rules that satisfy the following
		  criteria: (1) At least two antecedents. (2) Confidence > 50%. (3) Select one
		  rule with highest confidence
		   **/
        SPR spr = new SPR(sprInput1, sprInput2, sprOutput1, sprOutput2);
        spr.execute();

       
        long endTime = System.nanoTime();
        long totalDuration = (endTime-startTime)/1000000;

        System.out.println("\n\nSPR has successfully finished in "+totalDuration+" (ms)\n");
    }

}
