package WFPPM;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Scanner;
import java.util.Set;
import java.util.TreeMap;
import java.util.TreeSet;
import java.util.stream.Collectors;

/*The WFPPM module (run using this class WFPPM) takes in the output of the SHOD, CPS, SPR modules as well as the raw input click and purchase data*/
public class WFPPM{
	
	private String inputPath1;
	private String inputPath2;
	private String inputPath3;
	private String inputPath4;
	private String inputPath5;
	private String inputPath6;
	private int numOfUsersClicked; //as of now it is 100, above 100 will take longer as the output size is larger
	private String outputPath1;
	private String outputPath2; //output used in remaining modules
	 
	//default constructor
	public WFPPM() {
		this.inputPath1 = null;
		this.inputPath2 = null;
		this.inputPath3 = null;
		this.inputPath4 = null;
		this.inputPath5 = null;
		this.inputPath6 = null;
		this.outputPath1 = null;
		this.outputPath2 = null;
		numOfUsersClicked = 0;
	}
	
	//initializing constructor
	public WFPPM(String inputPath1, String inputPath2, String inputPath3, String inputPath4, String inputPath5, String inputPath6,  int numOfUsersClicked, String outputPath1, String outputPath2) {
		this.inputPath1 = inputPath1;
		this.inputPath2 = inputPath2;
		this.inputPath3 = inputPath3;
		this.inputPath4 = inputPath4;
		this.inputPath5 = inputPath5;
		this.inputPath6 = inputPath6;
		this.outputPath1 = outputPath1;
		this.outputPath2 = outputPath2;
		this.numOfUsersClicked = numOfUsersClicked;
	}
	
	
	private List<String> readSequence(Scanner sc) {
		List<String> sequences = new ArrayList<>();
		
		while(sc.hasNextLine()) 
			sequences.add(sc.nextLine().replaceAll("\"",""));
		
		return sequences;
	}
	
	
	private List<Double> readCPS(Scanner sc) {
		List<Double> CPSscore= new ArrayList<>();
		while(sc.hasNextLine()) {
			CPSscore.add(Double.parseDouble(sc.nextLine()));
		}
		return CPSscore;
		
	}
	
	
	private List<String> ComputeWFPPM(List<String> purchaseSequences,List<Double> CPSscores, List<String> uniqueItems) throws IOException {
		List<String> results = new ArrayList<String>();
		for(String item: uniqueItems) {
			int ItemCounter=0;
			double totalScore=0;
			Iterator<String> itl1=purchaseSequences.iterator();
			Iterator<Double> itl2=CPSscores.iterator();
			while(itl1.hasNext() && itl2.hasNext()) {
				double score=itl2.next();
				List<String> items = new ArrayList<String>(Arrays.stream(itl1.next().split(",")).collect(Collectors.toList()));
				for(String sequenceItem: items) {
					if(sequenceItem.equals(item)) {
						ItemCounter++;
						totalScore+=score;
					}
				}
			}
			results.add(item + " " + totalScore/ItemCounter);
		}
		return results;
	}

	
	private void printToFile(List<String> results) {
		try {
			BufferedWriter bw = new BufferedWriter(new FileWriter(outputPath1));
			results.forEach(result -> {try {
				bw.write(result + "\n");
			} catch (IOException e) {
				e.printStackTrace();
			}});
			 bw.close();
		} catch (IOException e1) {
			e1.printStackTrace();
		}
	}
	
	
	private List<String> readUniqueItems(Scanner sc) {
		List<String> uniqueItems = new ArrayList<>();
		
		while(sc.hasNextLine())
			uniqueItems.add(sc.nextLine());
		
		return uniqueItems;
	}
	
	
	private List<String> readUsers(Scanner sc){
		Set<String> users = new TreeSet<>();
		while(sc.hasNextLine()) {
			String[] list = sc.nextLine().split("\t");
			users.add(list[0]);
		}
		return users.stream().collect(Collectors.toList());
	}
	
	
	private List<String> getUsersThatOnlyClicked(Scanner usersThatClicked, List<String> usersThatPurchased){
		Set<String> usersThatOnlyClicked = new TreeSet<>();
		while(usersThatClicked.hasNextLine() && usersThatOnlyClicked.size() < numOfUsersClicked) {
			String[] user = usersThatClicked.nextLine().split("\t");
			
			if(usersThatPurchased.contains(user[0]) == false && usersThatOnlyClicked.contains(user[0]) == false) {
				usersThatOnlyClicked.add(user[0]);
			}
		}
		return usersThatOnlyClicked.stream().collect(Collectors.toList());
	}
	
	
	private Map<Integer, Map<String, Double>> readMatrix(Scanner sc){
		Map<Integer, Map<String, Double>> userItemFrequencyMatrix = new TreeMap<Integer, Map<String, Double>>();
		while(sc.hasNextLine()) {
			String[] purchase = sc.nextLine().split(" ");
			userItemFrequencyMatrix.computeIfAbsent(Integer.parseInt(purchase[0]), k -> new TreeMap<>()).put(purchase[1], Double.parseDouble(purchase[2]));
		}
		return userItemFrequencyMatrix;
	}
	
	
	private Map<Integer, Map<String, Double>> updateUserItemMatrix(Map<Integer, Map<String, Double>> userItemFrequencyMatrix, List<String> results, List<String> usersThatOnlyClicked) {
		usersThatOnlyClicked.forEach(userThatClicked -> {
			results.forEach(result -> {
				String[] list = result.split(" ");
				userItemFrequencyMatrix.computeIfAbsent(Integer.parseInt(userThatClicked), k -> new TreeMap<>()).put(list[0], Double.parseDouble(list[1]));
			
			});
		});
		return userItemFrequencyMatrix;
	}
	
	
	private void printMatrix(Map<Integer, Map<String, Double>> userItemFrequencyMatrix) {
		try {
			BufferedWriter bw = new BufferedWriter(new FileWriter(outputPath2));
			userItemFrequencyMatrix.forEach((k, v) -> {v.forEach((X,Y) -> {try {
				bw.write(k + " " + X + " " + Y + "\n");
			} catch (IOException e) {
				e.printStackTrace();
			}
			});});
			bw.close();
		} catch (IOException e1) {
			e1.printStackTrace();
		}
		
	}

	
	public void execute() {
		try {
			Scanner sc = new Scanner(new BufferedReader(new FileReader(inputPath1)));
		    List<String> purchase_sequences = readSequence(sc);
		    sc.close();
		    sc = new Scanner(new BufferedReader(new FileReader(inputPath2)));
		    List<Double> CPSscores = readCPS(sc);
		    sc.close();
		    sc = new Scanner(new BufferedReader(new FileReader(inputPath3)));
		    List<String> uniqueItems = readUniqueItems(sc);
		    sc.close();
			sc = new Scanner(new BufferedReader(new FileReader(inputPath4)));
		    List<String> users = readUsers(sc);
		    sc.close();
		    sc = new Scanner(new BufferedReader(new FileReader(inputPath5)));
		    List<String> usersThatOnlyClicked = getUsersThatOnlyClicked(sc, users);
		    sc.close();
		    sc = new Scanner(new BufferedReader(new FileReader(inputPath6)));
		    Map<Integer, Map<String, Double> > userItemFrequencyMatrix = readMatrix(sc);
		    sc.close();
		    
		    List<String> results = ComputeWFPPM(purchase_sequences, CPSscores, uniqueItems);
		    printToFile(results);
		    userItemFrequencyMatrix = updateUserItemMatrix(userItemFrequencyMatrix, results, usersThatOnlyClicked);
		    printMatrix(userItemFrequencyMatrix);
		    
		} catch (IOException e) {
			e.printStackTrace();
		}	   
	}
}