/*****************************************************************************************************************************************************************************
 * 
 * 1. INPUT:
 *      (1) There is six required files required for this system to run. You will need the data produced by the SHOD, SPR, CP

        (3) Adjust the filepaths at line 53.

        String directoryPath = "/home/woddlab/HSPREC_codes/Datasets/";

    2. HOW TO RUN:
        (1) To run the system you will either have to set this up in an IDE or use the command line, whichever is more suitable for your situation

        IDE SETUP:
            (1) Launch an IDE of your choice
            (2) Click on "file" and then open an existing java project and select the HSPREC folder.
            (3) Once the project is loaded in, find the WFPPMRunner and make the necessary adjustments so that the system works on your machine or directory.
                (Change the directory path)
            (4) Adjust the value at 84. This value will add that amount of user who did not purchase and only clicked. Use a simple program to determine this number. Change this value as you need
            (5) Now build and run the program!
        
        COMMAND LINE SETUP:
            (1) Open WFPPMRunner.java with a text editor and make the necessary adjustments so that the system works on your machine or directory.
                (Change the directory path)
            (2) Adjust the value at 84. This value will add that amount of user who did not purchase and only clicked. Use a simple program to determine this number. Change this value as you need
            (3) Open up the command line and make your way to where the WFPPM system is located.
            (4) Once you are at its location compile WFPPMRunner.java
                javac WFPPMRunner.java
            (5) Now run WFPPMRunner
                java WFPPMRunner

     3. DURING RUNTIME:
        (1) Once the program is running you will prompted to select either (1) paper_dataset or (2) Yoochoose
    
    4. OUTPUT:
        (1) Once the program is finished running it will produce 2 new .txt files in the data folder in results. These are the following files that should have been 
            produced:
                user_item_matrix_WFPPM_results.txt
                WFPPM_results.txt 
********************************************************************************************************************************************************************************* */
import WFPPM.*;

import java.util.Scanner;

/*
 * All code is done by Mustafa Sawalha, Xiaoshuai Geng, and Mustafa Alwindawi
 * CPS, WFPPM, and Normalization modules were solely made by Mustafa Alwindawi
 * Documented by Angel Veliz Castro. All documentation is from Raj's paper
 * */
public class WFPPMRunner {

    public static void main(String[] args) throws Exception {

        String directoryPath = "/home/woddlab/HSPREC_codes/HSPREC/Datasets/";

		boolean valid = true;
		Scanner sc = new Scanner(System.in);

		/*To make things easier instead of changing the directory path each time I mad a switch case that will add the final part depending on the users input*/
		while(valid) {
			String option;
			System.out.println("Which dataset do you want to run?\n1. paper_dataset\n2. Yoochoose\n");
			option = sc.nextLine();
			switch (option) {
				case "1":
					directoryPath = directoryPath + "paper_dataset/";
					valid = false;
					break;
				case "2":
					directoryPath = directoryPath + "Yoochoose/";
					valid = false;
					break;
				default:
					System.out.println("Invalid Input!");
			}
		}

       
        /* WFPPM inputs|outputs */
        String wfppmInput1 = directoryPath+"hsp_daily_sequential_db.txt";
        String wfppmInput2 = directoryPath+"cps_results.txt";
        String wfppmInput3 = directoryPath+"cps_uniqueItems.txt";
        String wfppmInput4 = directoryPath+"PurchaseData.txt";
        String wfppmInput5 = directoryPath+"ClickData.txt";
        String wfppmInput6 = directoryPath+"user_item_matrix_updated.txt";
        int wfppmInput7 = 1; //num of people that only clicked and did not purchase, added to the matrix. Change depending on your need
        String wfppmOutput1 = directoryPath+"WFPPM_results.txt";
        String wfppmOutput2 = directoryPath+"user_item_matrix_WFPPM_result.txt";

        /*********************************************************************************************************************************************************************************************************************************/

        long startTime = System.nanoTime();

        System.out.println("Starting WFPPM...");

		/*WFPPM: Weighted Frequent Purchase Pattern Miner
		  Weighted Frequent Purchase Pattern Miner(WFPPM) takes weighted purchase
		  pattern as input  and weighted purchase patterns is created
		  by assigning CPS (click sequence, purchase sequence) value to purchase patterns
		  in consequential bond to generate frequent purchase patterns with weight
		  under user specified minimum threshold as output.

		  Major steps of WFPPM are:

		  Count support of items: Count occurrence of items presented in weighted
		  purchase pattern

		  Calculate weight for individual item: Compute weight of individual item
		  from weighted purchase pattern using CPS module (Eq. (4)) (Can be found in the paper)

		  Test item weight with minimum threshold: Define minimum threshold
		  rating for user, here in our case, minimum threshold = 0.2. So, all rating of
		  item are frequent.
		*/
        WFPPM wfppm = new WFPPM(wfppmInput1, wfppmInput2, wfppmInput3, wfppmInput4, wfppmInput5, wfppmInput6, wfppmInput7, wfppmOutput1, wfppmOutput2);
        wfppm.execute();

       
        long endTime = System.nanoTime();
        long totalDuration = (endTime-startTime)/1000000;

        System.out.println("\n\nWFPPM has successfully finished in "+totalDuration+" (ms)\n");
    }

}
